// 3D Environment Laser Scanner (Arduino)
// By Callan Mackay
// Published on 22/02/2016

// Modified for the TB6612 stepper driver and NEMA 17 motor by Srisharan Shreedharan
// on 06/11/2016  dd/mm/yyyy
// Also added button feature to reset state

#include <Stepper.h>
# define STEPS 200  // number of motor steps 
Stepper stepper = Stepper(STEPS, 7, 8, 9 ,10);  // creating a structure to take in digital stepper inputs from Arduino
 
int dir = 1;  // direction, clockwise or counter clockwise 
int pot1 = 0; // speed
int pot2 = 1; // angleEnd
int button = 13; // pressing button resets state

void setup(){
  Serial.begin(9600);
  pinMode(pot1, INPUT);
  pinMode(pot2, INPUT);
  pinMode(button, INPUT);
  stepper.setSpeed(15);
}

int del;  // delay between each loop, determines the speed of the motor
float angle = 0;  // the current angle is held in this
int angleEnd; // the furthest angle that the motor will turn to 
int spd;  // the speed again, but just for displaying on the LCD screen

int pass = 0; // the number of times that the scanner has spun in each direction
int repeat = 2; // the number of times the scanner will turn in eech direction. Repeat of 2 returns it to it's starting position

void loop(){
  del = map(analogRead(pot1), 0, 1023, 500, 50);  // reading the pot1 value and mapping it to the delay
  spd = map(analogRead(pot1), 0, 1023, 1, 100); // reading the pot1 value and mapping it to the speed
  angleEnd = map(analogRead(pot2), 0, 1023, 0, 360);  //// reading the pot2 value and mapping it to the furthest angle

  if (pass < repeat){ // checking to see if the scanner needs to move
    if(pass%2 == 0){  // checking to see if the scanner has done an even number of passes
      dir = 1; // setting the steppers direction
      angle = angle + 1.8;  // increasing the current angle (quarter step)
    }
    else if(pass%2 == 1){ // checking to see if the scanner has done an odd number of passes
      dir = -1;  // setting the steppers direction
      angle = angle - 1.8;  // decreasing the current angle (quarter step)
      Serial.println(500.0);  // telling processing to STOP after the first pass. Allows the scanner to return to it's starting position after one pass
    }

    stepper.step(dir*1);  // Start telling the stepper driver to take a step
//    delayMicroseconds(300);   // tell it for a bit longer...
//    digitalWrite(stp, LOW);   // stop telling it so take a step
  }

  if ((angle >= angleEnd || angle <=0) && pass < repeat){   // has the stepper motor gone beyond the specified final angle, or beyond it's starting point?
    pass++;   // if so, then it's completed a pass so increase the number of passes it's done
  }

  Serial.println(angle);    // tell Processing what angle the motor's at

  //button state check
  if (digitalRead(button) && angle < 1){
    pass=0;
    angle=0;
  }
  
  delay(del);   // wait for a bit before starting it all again, this determines the speed of the motor  
  
}

